
/*!
*************************************************************************************
* \file  c3d_utilities_api.h
*
* \brief 
*        MFC SDK  Utilities layer API functions
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#ifndef _C3D_UTILITIES_
#define _C3D_UTILITIES_




/**
 * \fn void read_yuv_image(ImageData *p_img,ImageData *p_img_tmp, FILE *pf_input,uint8_t u8_packedUV);
 *
 * \brief Reads a yuv frame image from a 4:2:0 YUV file
 * 
 *
 * \param[out] p_img        Pointer to the image data struture 
 * \param[out] p_img_tmp    Pointer to the temporary image data struture used for u8_packedUV==1
 * \param[in]  pf_input      File Pointer to the input yuv file
 * \param[in]  u8_packedUV    Packed or Unpacked UV type

 *
 * \return
 */
int32_t read_yuv_image(ImageData *p_img,ImageData *p_img_tmp, FILE *pf_input,uint8_t u8_packedUV);

/**
 * \fn void write_yuv_image(ImageData *p_img,ImageData *p_img_tmp, FILE *pf_output,uint8_t u8_packedUV)
 *
 * \brief Writes a yuv frame image to a 4:2:0 YUV file
 * 
 *
 * \param[in]  p_img        Pointer to the image data struture 
 * \param[out] p_img_tmp    Pointer to the temporary image data struture used for u8_packedUV==1
 * \param[out] pf_output    File Pointer to the output yuv file
 * \param[in]  u8_packedUV    Packed or Unpacked UV type

 *
 * \return
 */
int32_t write_yuv_image(ImageData *p_img,ImageData *p_img_tmp, FILE *pf_output,uint8_t u8_packedUV);

/**
 * \fn int32_t allocate_image(
 *                               ImageData **p_img1,
 *                               ImageData *p_ref_image,
 *                               uint8_t    u8_packed_UV
 *                              )
 *
 * \brief Allocates memory for images by making API calls.
 *
 * \details Information about the frame (like frame width etc) is extracted from the reference image. The function
 * then makes an API call to the RPU kernel to allocate memory for the image data struture.
 *
 * \param[out]    p_img1                    Pointer to store the pointer to image 1
 * \param[in]    p_ref_image                Pointer to the reference image data structure. All the image info is
 *                                        extracted from this.
 * \param[in]    u8_packed_UV            UV packed alternatively or V followed by U
 * \return SUCCESS or FAILURE
 */
int32_t allocate_image(
                        ImageData **p_img1,
                        ImageData *p_ref_image,                       
                        uint8_t    u8_packed_UV
                       );

/**
 * \fn void generate_random_image(ImageData *p_img)
 *
 * \brief Generates a random source image
 *
 * \details Uses the system time to initialse the seed of the random number generator.
 *
 * \param[out] p_img    Pointer to the image data struture that is filled with a random image
 *
 * \return
 */
void generate_random_image(ImageData *p_img);

/**
 * \fn void copy_image_pixelData_only( 
 *                             ImageData* p_dst,
 *                       const ImageData* p_src
 *                      )  
 *
 * \brief Copies the source image to destination.
 *
 * \details Copies the Y,U and V buffers as well as other parameters to destination buffers. It is assumed that the 
 * memory has been allocated.(Doesnt change the destination Image Data Properties)
 *
 * \param[out]    p_dst    Pointer to destination image data structure.
 * \param[in]    p_src    Pointer to source image data structure. 
 *
 * \return None
 */
void copy_image_pixeldata_only(
                      ImageData *p_dst,
                const ImageData *p_src);


/**
 * \fn void copy_complete_imageData( 
 *                             ImageData* p_dst,
 *                       const ImageData* p_src
 *                      )  
 *
 * \brief Copies the source image to destination.
 *
 * \details Copies the Y,U and V buffers as well as other parameters to destination buffers. It is assumed that the 
 * memory has been allocated.
 *
 * \param[out]    p_dst    Pointer to destination image data structure.
 * \param[in]    p_src    Pointer to source image data structure. 
 *
 * \return None
 */
void copy_complete_ImageData(
                      ImageData *p_dst,
                const ImageData *p_src
               );

/**
 * \fn void pad_ImageData(
 *                      ImageData *p_dst,
 *                      uint16_t u16_orig_width,    
 *                      uint16_t u16_orig_height    
 *               )
 *
 * \brief Copies the source image to destination.
 *
 * \details Pads the X & Y boundaries of Image Data.
 *
 * \param[in/out]    p_dst    Pointer to destination image data structure.
 * \param[in]    u16_orig_width    Last column of the Image that is used to pad the rest of the columns. 
 * \param[in]    u16_orig_height    Last row of the Image that is used to pad the rest of the rows. 
 *
 * \return None
 */
void pad_ImageData(
                      ImageData *p_dst,
                      uint16_t u16_original_width,    
                      uint16_t u16_original_height    
               );




/**
 * \fn    void change_image_format(
 *                      ImageData *p_dst,
 *                const ImageData *p_src,
 *                      PICTURE_TYPE e_dst_picture_type 
 *               )
 *
 * \brief Copy Image Format (Dimensions and YUV Pointers) to a destination ImageData based on the picture_type.
 *            picture_type == FRAME ; Copy Image YUV pointers and Format as it is.
 *            picture_type == Top Field ; Copy Image YUV pointers, copy Format by doubling the stride , halving the height, halving the delimiter.
 *            picture_type == Bottom Field ; Copy Image YUV pointers with an offset of Stride , copy Format by doubling the stride , halving the height, halving the delimiter.
 *
 * \param[out]    p_dst    Pointer to destination image data structure.
 * \param[in]    p_src    Pointer to source image data structure.
 * \param[in]    e_dst_picture_type    Destination Picture Tye(Frame/topfield/bottom field)
 *
 * \return None
 */
void change_image_format(
                      ImageData *p_dst,
                const ImageData *p_src,
                      PICTURE_TYPE e_dst_picture_type 
               );


/**
 * \fn    void copy_image_dimensions(
 *                      ImageData *p_dst,
 *                const ImageData *p_src                      
 *               )
 *
 * \brief Copy Image Dimensions only from Source Image to Destination Image.
 *
 * \param[out]    p_dst    Pointer to destination image data structure.
 * \param[in]    p_src    Pointer to source image data structure. 
 *
 * \return None
 */
void copy_image_dimensions(
                      ImageData *p_dst,
                const ImageData *p_src                      
               );

/**
 * \fn    void print_rpu_header(RPUData *p_rpu_data);
 *
 * \brief Prints the RPU Header Information.
 * 
 * \param[in]    p_rpu_data    Pointer to RPU Header.
 *
 * \return None
 */
void print_rpu_header(RPUData *p_rpu_data);










/**
 * int setComponent_to_Dc(
 *                        ImageData    *p_dst,
 *                        uint16_t     u16_component_id,
 *                        uint16_t     u16_dc_value)
 *
 * \brief set a particular component values to dc value specified.
 * 
 * \param[out]    p_dst               Pointer to Image Data
 * \param[in]     u16_component_id    Component Id 0-luma,1-U,2-V;
 * \param[out]    u16_dc_value        Fixed DC value to set;
 *
 * \return SUCESS or FAILURE
 */
int32_t set_component_to_dc(
                        ImageData    *p_dst,
                        uint16_t     u16_component_id,
                        uint16_t     u16_dc_value);

#ifdef PRINT_RPU_INFO
#define PRINT_RPU_DATA(arg) printf arg
#else
#define PRINT_RPU_DATA(arg) 
#endif



#endif /* _C3D_UTILITIES_ */
